@@ -30,6 +30,7 @@ typedef struct _GtkHTMLEditPropertiesDialog   GtkHTMLEditPropertiesDialog;
 
 #include <gtkhtml.h>
 #include <bonobo.h>
+#include <libgnomeui/gnome-icon-theme.h>
 #include "persist-stream.h"
 #include "htmlgdkpainter.h"
 #include "search.h"
@@ -94,6 +95,7 @@ struct _GtkHTMLControlData {
 
 	CORBA_sequence_GNOME_Spell_Language *languages;
 	gboolean                block_language_changes;
+	gchar                  *language;
 	GNOME_Spell_Dictionary  dict;
 	EditorEngine           *editor_bonobo_engine;
 	BonoboObject           *persist_stream;
@@ -103,10 +105,13 @@ struct _GtkHTMLControlData {
 	GtkWidget *spell_dialog;
 	Bonobo_PropertyBag spell_control_pb;
 	gboolean has_spell_control;
+	gboolean has_spell_control_set;
 	gboolean spell_check_next;
 
 	GtkWidget *file_dialog;
 	gboolean file_html;
+
+	GnomeIconTheme *icon_theme;
 };
 
 GtkHTMLControlData * gtk_html_control_data_new       (GtkHTML *html, GtkWidget *vbox);
