@@ -1125,27 +1125,23 @@ get_item_marker_str (HTMLClueFlow *flow, gboolean ascii_only)
 static void
 draw_gt_line (HTMLObject *cur, HTMLPainter *p, gint offset, gint x, gint y)
 {
-	gint cy, w, a, d, line_offset = 0;
+	gint cy, w, a, d;
 
 	/* FIXME: cache items and glyphs? */
-	html_painter_calc_text_size (p, HTML_BLOCK_CITE, 
-				     strlen (HTML_BLOCK_CITE), NULL, NULL, NULL, 0, &line_offset,
-				     GTK_HTML_FONT_STYLE_SIZE_3, NULL,
+	html_painter_calc_text_size (p, HTML_BLOCK_CITE, strlen (HTML_BLOCK_CITE),
 				     &w, &a, &d);
 
 	cy = offset;
 	while (cy + a <= cur->ascent) {
 		/* FIXME: cache items and glyphs? */
-		html_painter_draw_text (p, x, y + cur->y - cy,
-					HTML_BLOCK_CITE, 1, NULL, NULL, NULL, 0, 0);
+		html_painter_draw_text (p, x, y + cur->y - cy, HTML_BLOCK_CITE, 1);
 		cy += a + d;
 	}
 
 	cy = - offset + a + d;
 	while (cy + d <= cur->descent) {
 		/* FIXME: cache items and glyphs? */
-		html_painter_draw_text (p, x, y + cur->y + cy,
-					HTML_BLOCK_CITE, 1, NULL, NULL, NULL, 0, 0);
+		html_painter_draw_text (p, x, y + cur->y + cy, HTML_BLOCK_CITE, 1);
 		cy += a + d;
 	}
 }
@@ -1246,19 +1242,18 @@ draw_item (HTMLObject *self, HTMLPainter *painter, gint x, gint y, gint width, g
 
 	marker = get_item_marker_str (flow, HTML_IS_PLAIN_PAINTER (painter));
 	if (marker) {
-		gint width, len, line_offset = 0, asc, dsc;
+		gint width, len, asc, dsc;
 		
 		len   = g_utf8_strlen (marker, -1);
-		/* FIXME: cache items and glyphs? */
-		html_painter_calc_text_size (painter, marker, len, NULL, NULL, NULL, 0, &line_offset,
-					     html_clueflow_get_default_font_style (flow), NULL, &width, &asc, &dsc);
-		width += html_painter_get_space_width (painter, html_clueflow_get_default_font_style (flow), NULL);
 		html_painter_set_font_style (painter, html_clueflow_get_default_font_style (flow));
 		html_painter_set_font_face  (painter, NULL);
 		/* FIXME: cache items and glyphs? */
+		html_painter_calc_text_size (painter, marker, len, &width, &asc, &dsc);
+		width += html_painter_get_space_width (painter, html_clueflow_get_default_font_style (flow), NULL);
+		/* FIXME: cache items and glyphs? */
 		html_painter_draw_text (painter, self->x + first->x - width + tx,
 					self->y - self->ascent + first->y + ty,
-					marker, len, NULL, NULL, NULL, 0, 0);
+					marker, len);
 	}
 	g_free (marker);
 }
