@@ -1262,7 +1262,7 @@ use_pictograms (HTMLEngine *e)
 		html_cursor_forward_n (e->cursor, e, len);
 
 		filename = g_strconcat ("file://" ICONDIR "/", picto_images [-state - 1], NULL);
-		picto = html_image_new (e->image_factory, filename, NULL, NULL, -1, -1, FALSE, FALSE, 0, NULL,
+		picto = html_image_new (html_engine_get_image_factory (e), filename, NULL, NULL, -1, -1, FALSE, FALSE, 0, NULL,
 					HTML_VALIGN_MIDDLE, FALSE);
 		html_image_set_alt (HTML_IMAGE (picto), alt);
 		html_object_set_data (HTML_OBJECT (picto), "picto", alt);
@@ -1272,13 +1272,13 @@ use_pictograms (HTMLEngine *e)
 }
 
 void
-html_engine_insert_text_with_extra_attributes (HTMLEngine *e, const gchar *text, guint len, PangoAttrList *attrs)
+html_engine_insert_text_with_extra_attributes (HTMLEngine *e, const gchar *text, gint len, PangoAttrList *attrs)
 {
 	gchar *nl;
 	gint alen;
+	gsize bytes;
 
-	if (len == -1)
-		len = g_utf8_strlen (text, -1);
+	bytes = html_text_sanitize (&text, &len);
 	if (!len)
 		return;
 
@@ -1287,7 +1287,7 @@ html_engine_insert_text_with_extra_attributes (HTMLEngine *e, const gchar *text,
 	gtk_html_editor_event_command (e->widget, GTK_HTML_COMMAND_INSERT_PARAGRAPH, TRUE);
 
 	do {
-		nl   = g_utf8_strchr (text, -1, '\n');
+		nl   = memchr (text, '\n', bytes);
 		alen = nl ? g_utf8_pointer_to_offset (text, nl) : len;
 		if (alen) {
 			HTMLObject *o;
@@ -1318,7 +1318,8 @@ html_engine_insert_text_with_extra_attributes (HTMLEngine *e, const gchar *text,
 		}
 		if (nl) {
 			html_engine_insert_empty_paragraph (e);
-			len -= g_utf8_pointer_to_offset (text, nl) + 1;
+			len -= alen + 1;
+			bytes -= (nl - text) + 1;
 			text = nl + 1;
 		}
 	} while (nl);
@@ -1326,7 +1327,7 @@ html_engine_insert_text_with_extra_attributes (HTMLEngine *e, const gchar *text,
 }
 
 void
-html_engine_insert_text (HTMLEngine *e, const gchar *text, guint len)
+html_engine_insert_text (HTMLEngine *e, const gchar *text, gint len)
 {
 	html_engine_insert_text_with_extra_attributes (e, text, len, NULL);
 }
@@ -1351,6 +1352,27 @@ html_engine_paste_text (HTMLEngine *e, const gchar *text, guint len)
 	html_engine_paste_text_with_extra_attributes (e, text, len, NULL);
 }
 
+void
+html_engine_paste_link (HTMLEngine *e, const char *text, int len, const char *complete_url)
+{
+	char *url, *target;
+
+	if (len == -1)
+		len = g_utf8_strlen (text, -1);
+
+	url = g_strdup (complete_url);
+	target = strrchr (url, '#');
+	if (target) {
+		*target = 0;
+		target ++;
+	}
+
+	html_engine_paste_text (e, text, len);
+	html_text_add_link (HTML_TEXT (e->cursor->object), e, url, target, e->cursor->offset - len, e->cursor->offset);
+
+	g_free (url);
+}
+
 void
 html_engine_delete_container (HTMLEngine *e)
 {
