@@ -140,7 +140,8 @@ draw_rect (HTMLPainter *painter,
 static guint
 get_page_width (HTMLPainter *painter, HTMLEngine *e)
 {
-	return MIN (72 * html_painter_get_space_width (painter, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED, NULL),
+	return MIN (72 * MAX (html_painter_get_space_width (painter, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED, NULL),
+			      html_painter_get_e_width (painter, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED, NULL)),
 		    html_engine_get_view_width (e)) + (html_engine_get_left_border (e) + html_engine_get_right_border (e));
 }
 
@@ -201,8 +202,6 @@ html_plain_painter_new (GtkWidget *widget, gboolean double_buffer)
 	new = g_object_new (HTML_TYPE_PLAIN_PAINTER, NULL);
 	html_painter_set_widget (HTML_PAINTER (new), widget);
 	HTML_GDK_PAINTER (new)->double_buffer = double_buffer;
-	HTML_GDK_PAINTER (new)->pc = gtk_widget_get_pango_context (widget);
-	g_object_ref (HTML_GDK_PAINTER (new)->pc);
 
 	return HTML_PAINTER (new);
 }
