@@ -134,7 +134,7 @@ calc_min_width (HTMLObject *o,
 		HTMLPainter *painter)
 {
 	if (HTML_TABLE_CELL (o)->no_wrap)
-		return MAX ((* HTML_OBJECT_CLASS (parent_class)->calc_preferred_width) (o, painter),
+		return MAX ((* HTML_OBJECT_CLASS (parent_class)->calc_min_width) (o, painter),
 			    o->flags & HTML_OBJECT_FLAG_FIXEDWIDTH
 			    ? HTML_TABLE_CELL (o)->fixed_width * html_painter_get_pixel_size (painter)
 			    : 0);
