@@ -83,6 +83,9 @@ html_text_pango_info_new (gint n)
 	pi->n = n;
 	pi->entries = g_new0 (HTMLTextPangoInfoEntry, n);
 	pi->attrs = NULL;
+	pi->have_font = FALSE;
+	pi->font_style = GTK_HTML_FONT_STYLE_DEFAULT;
+	pi->face = NULL;
 
 	return pi;
 }
@@ -97,6 +100,7 @@ html_text_pango_info_destroy (HTMLTextPangoInfo *pi)
 		g_free (pi->entries [i].widths);
 	}
 	g_free (pi->attrs);
+	g_free (pi->face);
 	g_free (pi);
 }
 
@@ -907,22 +911,12 @@ void
 html_text_calc_text_size (HTMLText *t, HTMLPainter *painter,
 			  gint start_byte_offset,
 			  guint len, HTMLTextPangoInfo *pi, GList *glyphs, gint *line_offset,
-			  GtkHTMLFontStyle font_style,
-			  HTMLFontFace *face,
 			  gint *width, gint *asc, gint *dsc)
 {
-		PangoAttrList *attrs = NULL;
 		char *text = t->text + start_byte_offset;
 
-		if (HTML_IS_PRINTER (painter)) {
-			attrs = html_text_get_attr_list (t, start_byte_offset, start_byte_offset + (g_utf8_offset_to_pointer (text, len) - text));
-		}
-		
-		html_painter_calc_text_size (painter, text, len, pi, attrs, glyphs,
-					     start_byte_offset, line_offset, font_style, face, width, asc, dsc);
-
-		if (attrs)
-			pango_attr_list_unref (attrs);
+		html_painter_calc_entries_size (painter, text, len, pi, glyphs,
+						line_offset, width, asc, dsc);
 }
 
 gint
@@ -1265,6 +1259,9 @@ html_text_get_pango_info (HTMLText *text, HTMLPainter *painter)
 		pango_attr_list_unref (attrs);
 
 		text->pi = html_text_pango_info_new (g_list_length (items));
+		text->pi->have_font = TRUE;
+		text->pi->font_style = html_text_get_font_style (text);
+		text->pi->face = g_strdup (text->face);
 
 		for (i = 0, cur = items; i < text->pi->n; i ++, cur = cur->next)
 			text->pi->entries [i].item = (PangoItem *) cur->data;
@@ -2379,13 +2376,62 @@ html_text_class_init (HTMLTextClass *klass,
 	parent_class = &html_object_class;
 }
 
-static gint
-text_len (const gchar **str, gint len)
+static gchar *
+offset_to_pointer_validated (const gchar *str, glong offset, gint *chars_out)
 {
-	if (g_utf8_validate (*str, -1, NULL))
-		return len != -1 ? len : g_utf8_strlen (*str, -1);
-	else {
+	const gchar *s = str;
+	glong chars = 0;
+
+	if (offset < 0) {
+		while (*s) {
+			gunichar wc = g_utf8_get_char_validated (s, -1);
+			if (wc == (gunichar)-1 || wc == (gunichar)-2)
+				return NULL;
+			s = g_utf8_next_char (s);
+			chars++;
+		}
+		
+	} else {
+		while (offset-- && *s) {
+			gunichar wc = g_utf8_get_char_validated (s, -1);
+			if (wc == (gunichar)-1 || wc == (gunichar)-2)
+				return NULL;
+			s = g_utf8_next_char (s);
+			chars++;
+		}
+	}
+
+	*chars_out = chars;
+	
+	return (gchar *)s;
+}
+
+/**
+ * html_text_sanitize:
+ * @str: text string (in/out)
+ * @len: length of text, in characters (in/out). (A value of
+ *       -1 on input means to use all characters in @str)
+ * 
+ * Validates a UTF-8 string up to the given number of characters;
+ * if the string is invalid, on output, "[?]" will be stored in
+ * @str and 3 in @len, otherwise @str will be left unchanged,
+ * and @len will be left unchanged if non-negative, otherwise
+ * replaced with the number of characters in @str.
+ * 
+ * Return value: number of bytes in the output value of @str
+ **/
+gsize
+html_text_sanitize (const gchar **str, gint *len)
+{
+	g_return_val_if_fail (str != NULL, 0);
+	g_return_val_if_fail (len != NULL, 0);
+	
+	gchar *end = offset_to_pointer_validated (*str, *len, len);
+	if (end) {
+		return end - *str;
+	} else {
 		*str = "[?]";
+		*len = 3;
 		return 3;
 	}
 }
@@ -2402,9 +2448,9 @@ html_text_init (HTMLText *text,
 
 	html_object_init (HTML_OBJECT (text), HTML_OBJECT_CLASS (klass));
 
-	text->text_len      = text_len (&str, len);
-	text->text_bytes    = strlen (str);
-	text->text          = g_strndup (str, g_utf8_offset_to_pointer (str, text->text_len) - str);
+	text->text_bytes    = html_text_sanitize (&str, &len);
+	text->text_len      = len;
+	text->text          = g_memdup (str, text->text_bytes + 1);
 	text->font_style    = font_style;
 	text->face          = NULL;
 	text->color         = color;
@@ -2482,9 +2528,9 @@ void
 html_text_set_text (HTMLText *text, const gchar *new_text)
 {
 	g_free (text->text);
-	text->text_len = text_len (&new_text, -1);
-	text->text = g_strdup (new_text);
-	text->text_bytes = strlen (text->text);
+	text->text_len = -1;
+	text->text_bytes = html_text_sanitize (&new_text, &text->text_len);
+	text->text = g_memdup (new_text, text->text_bytes);
 	html_object_change_set (HTML_OBJECT (text), HTML_CHANGE_ALL);
 }
 
@@ -2739,12 +2785,18 @@ void
 html_text_append (HTMLText *text, const gchar *str, gint len)
 {
 	gchar *to_delete;
+	guint bytes;
 
 	to_delete       = text->text;
-	text->text_len += text_len (&str, len);
-	text->text_bytes += strlen (str);
-	text->text      = g_strconcat (to_delete, str, NULL);
-
+	bytes = html_text_sanitize (&str, &len);
+	text->text_len += len;
+	text->text      = g_malloc (text->text_bytes + bytes + 1);
+	
+	memcpy (text->text, to_delete, text->text_bytes);
+	memcpy (text->text + text->text_bytes, str, bytes);
+	text->text_bytes += bytes;
+	text->text[text->text_bytes] = '\0';
+	
 	g_free (to_delete);
 
 	html_object_change_set (HTML_OBJECT (text), HTML_CHANGE_ALL);
@@ -3244,7 +3296,9 @@ html_text_change_attrs (PangoAttrList *attr_list, GtkHTMLFontStyle style, HTMLEn
 
 	if (!avoid_default_size
 	    || (((style & GTK_HTML_FONT_STYLE_SIZE_MASK) != GTK_HTML_FONT_STYLE_DEFAULT)
-		&& ((style & GTK_HTML_FONT_STYLE_SIZE_MASK) != GTK_HTML_FONT_STYLE_SIZE_3))) {
+		&& ((style & GTK_HTML_FONT_STYLE_SIZE_MASK) != GTK_HTML_FONT_STYLE_SIZE_3))
+	    || ((style & GTK_HTML_FONT_STYLE_FIXED) &&
+		e->painter->font_manager.fix_size != e->painter->font_manager.var_size)) {
 		attr = html_pango_attr_font_size_new (style);
 		html_pango_attr_font_size_calc ((HTMLPangoAttrFontSize *) attr, e);
 		attr->start_index = start_index;
@@ -3256,7 +3310,7 @@ html_text_change_attrs (PangoAttrList *attr_list, GtkHTMLFontStyle style, HTMLEn
 void
 html_text_set_style_in_range (HTMLText *text, GtkHTMLFontStyle style, HTMLEngine *e, gint start_index, gint end_index)
 {
-	html_text_change_attrs (text->attr_list, style, e, start_index, end_index, FALSE);
+	html_text_change_attrs (text->attr_list, style, e, start_index, end_index, TRUE);
 }
 
 void
