@@ -45,13 +45,17 @@ struct _Link {
 
 struct _HTMLTextPangoInfoEntry {
 	PangoItem *item;
-	PangoLogAttr *attrs;
 	PangoGlyphUnit *widths;
 };
 
 struct _HTMLTextPangoInfo {
 	HTMLTextPangoInfoEntry *entries;
+	PangoLogAttr *attrs;
 	gint n;
+	
+	gboolean have_font;
+	GtkHTMLFontStyle font_style;
+	HTMLFontFace *face;
 };
 
 struct _HTMLPangoAttrFontSize {
@@ -161,6 +165,7 @@ gint              html_text_text_line_length             (const gchar        *te
 							  gint               *tabs);
 gint              html_text_calc_part_width              (HTMLText           *text,
 							  HTMLPainter        *painter,
+							  char               *start,
 							  gint                offset,
 							  gint                len,
 							  gint               *asc,
@@ -210,6 +215,7 @@ void              html_text_add_link_full                (HTMLText           *te
 							  gint                end_index,
 							  gint                start_offset,
 							  gint                end_offset);
+void              html_text_remove_links                 (HTMLText           *text);
 gboolean          html_text_get_link_rectangle           (HTMLText           *text,
 							  HTMLPainter        *painter,
 							  gint                offset,
@@ -266,6 +272,9 @@ void              html_text_set_color_in_range           (HTMLText           *te
 void              html_text_set_color                    (HTMLText           *text,
 							  HTMLColor          *color);
 
+gsize             html_text_sanitize                     (const gchar       **str,
+							  gint               *len);
+
 Link     *html_link_new                 (gchar *url,
 					 gchar *target,
 					 guint  start_index,
@@ -298,20 +307,19 @@ PangoAttrList     *html_text_get_attr_list         (HTMLText              *text,
 						    gint                   end_index);
 void               html_text_calc_text_size        (HTMLText              *t,
 						    HTMLPainter           *painter,
-						    const gchar           *text,
+						    gint                   start_byte_offset,
 						    guint                  len,
 						    HTMLTextPangoInfo     *pi,
 						    GList                 *glyphs,
-						    gint                   start_byte_offset,
 						    gint                  *line_offset,
-						    GtkHTMLFontStyle       font_style,
-						    HTMLFontFace          *face,
 						    gint                  *width,
 						    gint                  *asc,
 						    gint                  *dsc);
-gboolean           html_text_is_line_break         (PangoItem             *item,
-						    PangoLogAttr          *attrs,
-						    int                    offset);
+
+gboolean  html_text_is_line_break                (PangoLogAttr  attr);
+void      html_text_remove_unwanted_line_breaks  (char         *s,
+						  int           len,
+						  PangoLogAttr *attrs);
 
 typedef HTMLObject * (* HTMLTextHelperFunc)       (HTMLText *, gint begin, gint end);
 HTMLObject *html_text_op_copy_helper    (HTMLText           *text,
